@@ -19,112 +19,11 @@
  *
  */
 
-#include <errno.h>
-#include <time.h>
-
 #include <osmocom/ctrl/control_cmd.h>
 
-#include <osmocom/vty/command.h>
-#include <osmocom/vty/misc.h>
-
 #include <osmocom/bsc/ctrl.h>
-#include <osmocom/bsc/osmo_bsc_rf.h>
 #include <osmocom/bsc/bts.h>
-#include <osmocom/bsc/ipaccess.h>
-#include <osmocom/bsc/chan_alloc.h>
 #include <osmocom/bsc/abis_nm.h>
-#include <osmocom/bsc/neighbor_ident.h>
-#include <osmocom/bsc/system_information.h>
-
-/*********************
- * TS_NODE
- *********************/
-
-static int verify_ts_hopping_arfcn_add(struct ctrl_cmd *cmd, const char *value, void *_data)
-{
-	int64_t arfcn;
-	enum gsm_band unused;
-	if (osmo_str_to_int64(&arfcn, value, 10, 0, 1024) < 0)
-		return 1;
-	if (gsm_arfcn2band_rc(arfcn, &unused) < 0) {
-		return 1;
-	}
-	return 0;
-}
-static int set_ts_hopping_arfcn_add(struct ctrl_cmd *cmd, void *data)
-{
-	struct gsm_bts_trx_ts *ts = cmd->node;
-	int arfcn = atoi(cmd->value);
-
-	bitvec_set_bit_pos(&ts->hopping.arfcns, arfcn, ONE);
-
-	/* Update Cell Allocation (list of all the frequencies allocated to a cell) */
-	if (generate_cell_chan_alloc(ts->trx->bts) != 0) {
-		bitvec_set_bit_pos(&ts->hopping.arfcns, arfcn, ZERO); /* roll-back */
-		cmd->reply = "Failed to re-generate Cell Allocation";
-		return CTRL_CMD_ERROR;
-	}
-
-	cmd->reply = "OK";
-	return CTRL_CMD_REPLY;
-}
-/* Parameter format: "<arfcn>" */
-CTRL_CMD_DEFINE_WO(ts_hopping_arfcn_add, "hopping-arfcn-add");
-
-static int verify_ts_hopping_arfcn_del(struct ctrl_cmd *cmd, const char *value, void *_data)
-{
-	int64_t arfcn;
-	enum gsm_band unused;
-	if (strcmp(value, "all") == 0)
-		return 0;
-	if (osmo_str_to_int64(&arfcn, value, 10, 0, 1024) < 0)
-		return 1;
-	if (gsm_arfcn2band_rc(arfcn, &unused) < 0) {
-		return 1;
-	}
-	return 0;
-}
-static int set_ts_hopping_arfcn_del(struct ctrl_cmd *cmd, void *data)
-{
-	struct gsm_bts_trx_ts *ts = cmd->node;
-	bool all = (strcmp(cmd->value, "all") == 0);
-	int arfcn;
-
-	if (all) {
-		bitvec_zero(&ts->hopping.arfcns);
-	} else {
-		arfcn = atoi(cmd->value);
-		bitvec_set_bit_pos(&ts->hopping.arfcns, arfcn, ZERO);
-	}
-
-	/* Update Cell Allocation (list of all the frequencies allocated to a cell) */
-	if (generate_cell_chan_alloc(ts->trx->bts) != 0) {
-		if (!all)
-			bitvec_set_bit_pos(&ts->hopping.arfcns, arfcn, ONE); /* roll-back */
-		cmd->reply = "Failed to re-generate Cell Allocation";
-		return CTRL_CMD_ERROR;
-	}
-
-	cmd->reply = "OK";
-	return CTRL_CMD_REPLY;
-}
-/* Parameter format: "(<arfcn>|all)" */
-CTRL_CMD_DEFINE_WO(ts_hopping_arfcn_del, "hopping-arfcn-del");
-
-
-static int bsc_bts_trx_ts_ctrl_cmds_install(void)
-{
-	int rc = 0;
-
-	rc |= ctrl_cmd_install(CTRL_NODE_TS, &cmd_ts_hopping_arfcn_add);
-	rc |= ctrl_cmd_install(CTRL_NODE_TS, &cmd_ts_hopping_arfcn_del);
-
-	return rc;
-}
-
-/*********************
- * TRX_NODE
- *********************/
 
 static int get_trx_rf_locked(struct ctrl_cmd *cmd, void *data)
 {
@@ -198,6 +97,51 @@ static int set_trx_max_power(struct ctrl_cmd *cmd, void *_data)
 }
 CTRL_CMD_DEFINE(trx_max_power, "max-power-reduction");
 
+char *trx_lchan_dump_full_ctrl(const void *t, struct gsm_bts_trx *trx)
+{
+	int ts_nr;
+	bool first_ts = true;
+	char *ts_dump, *dump;
+
+	dump = talloc_strdup(t, "");
+	if (!dump)
+		return NULL;
+
+	for (ts_nr = 0; ts_nr < TRX_NR_TS; ts_nr++) {
+		ts_dump = ts_lchan_dump_full_ctrl(t, &trx->ts[ts_nr]);
+		if (!ts_dump)
+			return NULL;
+		if (!strlen(ts_dump))
+			continue;
+		dump = talloc_asprintf_append(dump, first_ts ? "%s" : "\n%s", ts_dump);
+		if (!dump)
+			return NULL;
+		first_ts = false;
+	}
+
+	return dump;
+}
+
+/* Return full information about all logical channels in a TRX.
+ * format: bts.<0-255>.trx.<0-255>.show-lchan.full
+ * result format: New line delimited list of <bts>,<trx>,<ts>,<lchan>,<type>,<connection>,<state>,<last error>,<bs power>,
+ *  <ms power>,<interference dbm>, <interference band>,<channel mode>,<imsi>,<tmsi>,<ipa bound ip>,<ipa bound port>,
+ *  <ipa bound conn id>,<ipa conn ip>,<ipa conn port>,<ipa conn speech mode>
+ */
+static int get_trx_show_lchan_full(struct ctrl_cmd *cmd, void *data)
+{
+	struct gsm_bts_trx *trx = cmd->node;
+
+	cmd->reply = trx_lchan_dump_full_ctrl(cmd, trx);
+	if (!cmd->reply) {
+		cmd->reply = "OOM";
+		return CTRL_CMD_ERROR;
+	}
+
+	return CTRL_CMD_REPLY;
+}
+CTRL_CMD_DEFINE_RO(trx_show_lchan_full, "show-lchan full");
+
 int bsc_bts_trx_ctrl_cmds_install(void)
 {
 	int rc = 0;
@@ -205,6 +149,7 @@ int bsc_bts_trx_ctrl_cmds_install(void)
 	rc |= ctrl_cmd_install(CTRL_NODE_TRX, &cmd_trx_max_power);
 	rc |= ctrl_cmd_install(CTRL_NODE_TRX, &cmd_trx_arfcn);
 	rc |= ctrl_cmd_install(CTRL_NODE_TRX, &cmd_trx_rf_locked);
+	rc |= ctrl_cmd_install(CTRL_NODE_TRX, &cmd_trx_show_lchan_full);
 
 	rc |= bsc_bts_trx_ts_ctrl_cmds_install();
 
